from sqlalchemy import Column, Integer, String
from petisco.persistence.sqlalchemy.sqlalchemy_persistence import SqlAlchemyPersistence

Base = SqlAlchemyPersistence.get_instance().sources["petisco"]["base"]


class UserModel(Base):
    __tablename__ = "User"

    id = Column("id", Integer, primary_key=True)
    user_id = Column("user_id", String(12))
    client_id = Column("client_id", String(50))
    name = Column("name", String(50))



from petisco import RabbitMqConnector, RabbitMQEventPublisher
from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqEventPublisherMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMQEventPublisher(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, f"{DEFAULT_SERVICE}-event"
        )



def get_event_binding_key(organization: str, service: str):
    """
    acme.onboarding.1.event.user.created
      |       |      |        |      |-> Action (past verb)
      |       |      |        |-> domain entity
      |       |      |-> version
      |       |-> service
      |-> organization
    """
    return f"{organization}.{service}.*.event.#"



from petisco import LogMessage
from petisco.logger.interface_logger import ILogger


class FakeLogger(ILogger):
    def __init__(self):
        self.logging_messages = []

    def log(self, logging_level, log_message: LogMessage):
        self.logging_messages.append((logging_level, log_message.to_dict()))

    def get_logging_messages(self):
        return self.logging_messages



from time import sleep

import pytest
from meiga import isSuccess, Result, Error, Failure

from petisco import Event, DEBUG
from petisco.event.chaos.domain.event_chaos_error import EventChaosError
from petisco.event.chaos.infrastructure.rabbitmq_event_chaos import RabbitMqEventChaos
from petisco.event.shared.domain.event_subscriber import EventSubscriber

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents
from tests.modules.unit.mocks.fake_logger import FakeLogger


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_consumer_react_to_default_no_chaos():

    spy = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos()
    consumer = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    assert len(logger.get_logging_messages()) == 1
    logging_message = logger.get_logging_messages()[0]
    assert logging_message[0] == DEBUG
    assert logging_message[1]["data"]["message"]["derived_action"] == {}


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_consumer_react_to_chaos_with_zero_probability():

    spy = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos(
        percentage_simulate_nack=0.0, percentage_simulate_failures=0.0
    )
    consumer = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    assert len(logger.get_logging_messages()) == 1
    logging_message = logger.get_logging_messages()[0]
    assert logging_message[0] == DEBUG
    assert logging_message[1]["data"]["message"]["derived_action"] == {}


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "max_retries_allowed,expected_number_event_consumed,chaos",
    [
        (5, 0, RabbitMqEventChaos(percentage_simulate_nack=1.0)),
        (1, 2, RabbitMqEventChaos(percentage_simulate_failures=1.0)),
        (1, 1, RabbitMqEventChaos(delay_before_even_handler_second=2.0)),
    ],
)
def test_should_consumer_react_to_chaos_inputs(
    max_retries_allowed, expected_number_event_consumed, chaos
):
    spy = SpyEvents()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_chaos(chaos, max_retries_allowed)
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()
    spy.assert_count_by_event_id(event.event_id, expected_number_event_consumed)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_consumer_react_to_chaos_with_nck_simulation_and_check_logger():

    spy = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos(percentage_simulate_nack=1.0)
    consumer = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    assert len(logger.get_logging_messages()) >= 200
    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message[0] == DEBUG
    assert first_logging_message[1]["meta"] == {
        "layer": "rabbitmq_event_consumer",
        "operation": "assert_consumer",
    }
    assert (
        first_logging_message[1]["data"]["message"]["chaos_action"] == "nck simulated"
    )


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_consumer_react_to_chaos_with_failure_simulation_and_check_logger():

    spy = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos(percentage_simulate_failures=1.0)
    consumer = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    assert_logger_represents_simulated_failure_scenario(logger, max_retries_allowed)


def assert_logger_represents_simulated_failure_scenario(logger, max_retries_allowed):
    assert len(logger.get_logging_messages()) == max_retries_allowed + 1
    for i in range(max_retries_allowed):
        redelivery_count = i + 1
        logging_message = logger.get_logging_messages()[i]

        assert logging_message[0] == DEBUG
        assert logging_message[1]["meta"] == {
            "layer": "rabbitmq_event_consumer",
            "operation": "assert_consumer",
        }
        assert logging_message[1]["data"]["message"]["result"] == Failure(
            EventChaosError(Exception())
        )

        derived_action = logging_message[1]["data"]["message"]["derived_action"]

        if i < 5:
            expected_derived_action = {
                "action": "send_to_retry",
                "exchange_name": "retry.alice.petisco",
                "routing_key": "retry.alice.petisco.1.event.user_created.assert_consumer",
                "headers": {
                    "queue": "alice.petisco.1.event.user_created.assert_consumer",
                    "redelivery_count": redelivery_count,
                },
            }
        else:
            expected_derived_action = {
                "action": "send_to_dead_letter",
                "exchange_name": "dead_letter.alice.petisco",
                "routing_key": "dead_letter.alice.petisco.1.event.user_created.assert_consumer",
                "headers": {
                    "queue": "alice.petisco.1.event.user_created.assert_consumer",
                    "redelivery_count": redelivery_count,
                },
            }

        if i > 0:
            derived_action["headers"].pop("x-death")
            derived_action["headers"].pop("x-first-death-exchange")
            derived_action["headers"].pop("x-first-death-queue")
            derived_action["headers"].pop("x-first-death-reason")

        assert derived_action == expected_derived_action


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_consumer_react_to_chaos_with_nck_simulation_and_send_event_to_dead_letter():
    spy = SpyEvents()
    spy_dead_letter = SpyEvents()
    spy_dead_letter_store = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    def assert_dead_letter_consumer(event: Event) -> Result[bool, Error]:
        spy_dead_letter.append(event)
        return isSuccess

    def assert_dead_letter_store_consumer(event: Event) -> Result[bool, Error]:
        spy_dead_letter_store.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_main_and_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos(percentage_simulate_nack=1.0)
    consumer_with_chaos = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer_with_chaos.add_subscribers(subscribers)
    consumer_with_chaos.start()
    sleep(1.0)
    consumer_with_chaos.stop()

    consumer_without_chaos = RabbitMqEventConsumerMother.default()
    consumer_without_chaos.add_handler_on_queue(
        "dead_letter.alice.petisco.1.event.user_created.assert_consumer",
        assert_dead_letter_consumer,
    )
    consumer_without_chaos.add_handler_on_queue(
        "dead_letter.store", assert_dead_letter_store_consumer
    )

    consumer_without_chaos.start()
    sleep(1.0)
    consumer_without_chaos.stop()

    configurer.clear()

    spy.assert_count_by_event_id(event.event_id, 0)  # Rejected before by Event Chaos
    spy_dead_letter.assert_count_by_event_id(event.event_id, 1)
    spy_dead_letter_store.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_store_consumer_react_to_chaos_with_nck_simulation_and_send_several_event_to_dead_letter():
    spy = SpyEvents()
    spy_dead_letter_store = SpyEvents()
    logger = FakeLogger()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    def assert_dead_letter_store_consumer(event: Event) -> Result[bool, Error]:
        spy_dead_letter_store.append(event)
        return isSuccess

    configurer = RabbitMqEventConfigurerMother.with_main_and_retry_ttl_10ms()
    configurer.configure()

    bus = RabbitMqEventBusMother.default()

    event_ids = []
    for _ in range(5):
        event = EventUserCreatedMother.random()
        event_ids.append(event.event_id)
        bus.publish(event)

    max_retries_allowed = 5
    chaos = RabbitMqEventChaos(percentage_simulate_nack=1.0)
    consumer_with_chaos = RabbitMqEventConsumerMother.with_chaos(
        chaos, max_retries_allowed, logger
    )
    consumer_with_chaos.add_handler_on_store(assert_consumer)
    consumer_with_chaos.start()
    sleep(1.0)
    consumer_with_chaos.stop()

    consumer_without_chaos = RabbitMqEventConsumerMother.default()
    consumer_without_chaos.add_handler_on_queue(
        "dead_letter.store", assert_dead_letter_store_consumer
    )

    consumer_without_chaos.start()
    sleep(1.0)
    consumer_without_chaos.stop()

    configurer.clear()

    for event_id in event_ids:
        spy.assert_count_by_event_id(event_id, 0)  # Rejected before by Event Chaos
        spy_dead_letter_store.assert_count_by_event_id(event_id, 1)



from pymongo import MongoClient
from pymongo.errors import ConnectionFailure

from petisco.persistence.interface_persistence_connector import IPersistenceConnector
from petisco.persistence.pymongo.pymongo_persistence import PyMongoPersistence
from petisco.persistence.pymongo.pymongo_persistence_config import (
    PyMongoPersistenceConfig,
)


class PyMongoPersistenceConnector(IPersistenceConnector):
    def __init__(self, config: PyMongoPersistenceConfig):
        self.config = config

    def execute(self):
        if not self.config.database:
            raise ValueError(
                "Petisco PyMongoPersistenceConnector is not configured correctly. "
                "Please check PyMongoPersistenceConfig and add database name"
            )

        mongo_client = MongoClient(
            f"mongodb://{self.config.host}:{self.config.port}/",
            username=self.config.user,
            password=self.config.password,
        )
        try:
            # The ismaster command is cheap and does not require auth.
            mongo_client.admin.command("ismaster")
        except ConnectionFailure:
            raise ConnectionError(
                "Petisco PyMongoPersistenceConnector can't connect to MongoDB. "
                "Please check PyMongoPersistenceConfig and add required values"
            )
        mongo_database = mongo_client[self.config.database]
        PyMongoPersistence(client=mongo_client, database=mongo_database)



import uuid
import validators

from petisco.domain.value_objects.value_object import ValueObjectError, ValueObject


class InvalidUuidError(ValueObjectError):
    def __init__(self, uuid_value: str):
        self.message = f"{self.__class__.__name__}: [uuid: {uuid_value}]"


class Uuid(ValueObject):
    @staticmethod
    def length():
        return 36

    def __init__(self, value: str):
        self.value = value
        super(Uuid, self).__init__()

    def __hash__(self):
        return hash(str(self))

    def __repr__(self):
        return f"[{self.__class__.__name__}: {self.value}]"

    def __eq__(self, other):
        if issubclass(other.__class__, self.__class__) or issubclass(
            self.__class__, other.__class__
        ):
            return self.value == other.value
        else:
            return False

    def guard(self):
        self._ensure_is_valid_uuid(self.value)

    def _ensure_is_valid_uuid(self, value):
        if not validators.uuid(value):
            raise InvalidUuidError(value)

    @classmethod
    def generate(cls):
        return cls(str(uuid.uuid4()))



from time import sleep

import pytest
from meiga import isSuccess, Result, Error, isFailure, BoolResult

from petisco import Event, EventSubscriber

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_from_store_queue_from_rabbitmq():

    spy = SpyEvents()

    def assert_consumer_event_store(event: Event) -> BoolResult:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_event(event)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.default()

    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.start()

    sleep(1.0)
    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "max_retries_allowed,expected_number_event_consumed,simulated_results",
    [
        (0, 1, [isFailure]),
        (1, 2, [isFailure, isSuccess]),
        (2, 3, [isFailure, isFailure, isSuccess]),
        (3, 4, [isFailure, isFailure, isFailure, isSuccess]),
        (4, 5, [isFailure, isFailure, isFailure, isFailure, isSuccess]),
    ],
)
def test_should_publish_consume_and_retry_from_store_queue_from_rabbitmq(
    max_retries_allowed, expected_number_event_consumed, simulated_results
):
    spy = SpyEvents()

    def assert_consumer_event_store(event: Event) -> Result[bool, Error]:
        spy.append(event)
        result = simulated_results.pop(0)
        return result

    event = EventUserCreatedMother.random()

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(event)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_max_retries(max_retries_allowed)

    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.start()

    sleep(1.0)

    consumer.stop()

    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, expected_number_event_consumed)


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "max_retries_allowed,expected_number_event_consumed,simulated_results",
    [
        (0, 1, [isFailure]),
        (1, 2, [isFailure, isSuccess]),
        (2, 3, [isFailure, isFailure, isSuccess]),
        (3, 4, [isFailure, isFailure, isFailure, isSuccess]),
        (4, 5, [isFailure, isFailure, isFailure, isFailure, isSuccess]),
    ],
)
def test_should_publish_consume_and_retry_from_store_queue_not_affecting_other_queue_from_rabbitmq_when_fail_event_storer_consumer(
    max_retries_allowed, expected_number_event_consumed, simulated_results
):
    spy_consumer_event_store = SpyEvents()
    spy_consumer_handler = SpyEvents()

    def assert_consumer_event_store(event: Event) -> Result[bool, Error]:
        spy_consumer_event_store.append(event)
        result = simulated_results.pop(0)
        return result

    def assert_consumer_handler(event: Event) -> Result[bool, Error]:
        spy_consumer_handler.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer_handler],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_max_retries(max_retries_allowed)
    consumer.add_subscribers(subscribers)
    consumer.add_handler_on_store(assert_consumer_event_store)

    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy_consumer_event_store.assert_number_unique_events(1)
    spy_consumer_event_store.assert_first_event(event)
    spy_consumer_event_store.assert_count_by_event_id(
        event.event_id, expected_number_event_consumed
    )

    spy_consumer_handler.assert_number_unique_events(1)
    spy_consumer_handler.assert_first_event(event)
    spy_consumer_handler.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "max_retries_allowed,simulated_results",
    [
        (1, [isFailure, isSuccess, isSuccess]),
        (2, [isFailure, isFailure, isSuccess, isSuccess]),
        (3, [isFailure, isFailure, isFailure, isSuccess, isSuccess]),
        (4, [isFailure, isFailure, isFailure, isFailure, isSuccess, isSuccess]),
    ],
)
def test_should_publish_two_event_and_consume_from_store_queue_from_rabbitmq(
    max_retries_allowed, simulated_results
):

    spy = SpyEvents()

    def assert_consumer_event_store(event: Event) -> Result[bool, Error]:
        spy.append(event)
        result = simulated_results.pop(0)
        return result

    event_1 = EventUserCreatedMother.random()
    event_2 = EventUserCreatedMother.random()

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(event_1)
    configurer.configure_event(event_2)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event_1)
    bus.publish(event_2)

    consumer = RabbitMqEventConsumerMother.with_max_retries(max_retries_allowed)

    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.start()

    sleep(1.0)
    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(2)



from typing import Callable, Dict

from meiga import Result
from pika import BasicProperties
from pika.spec import Basic


class RabbitMqEventConsumerPrinter:
    def __init__(self, verbose: bool = False):
        self.verbose = verbose

    def print_received_message(
        self, method: Basic.Deliver, properties: BasicProperties, body: bytes
    ):
        if self.verbose:
            print(
                "\n#####################################################################################################################"
            )
            print(" [x] Received %r" % (body,))
            print(" [x] Properties %r" % (properties,))
            print(" [x] method %r" % (method,))

    def print_separator(self):
        if self.verbose:
            print(
                "#####################################################################################################################\n"
            )

    def print_context(self, handler: Callable, result: Result):
        if self.verbose:
            handler_name = getattr(handler, "__name__", repr(handler))
            handler_module = getattr(handler, "__module__") + "."

            print(f" [x] event_handler: {handler_module}{handler_name}")
            print(f" [x] result from event_handler: {result}")

    def print_action(self, action_name: str):
        if self.verbose:
            print(f" [>] {action_name}")

    def print_send_message_to(
        self, exchange_name: str, routing_key: str, headers: Dict
    ):
        if self.verbose:
            print(f" [>] send: [{exchange_name} |{routing_key}] -> {headers}")



from typing import Dict

from meiga import Result
from abc import ABCMeta, abstractmethod


class IEventChaos:

    __metaclass__ = ABCMeta

    @abstractmethod
    def info(self) -> Dict:
        raise NotImplementedError

    @abstractmethod
    def nack_simulation(self, *args, **kwargs) -> bool:
        raise NotImplementedError

    @abstractmethod
    def simulate_failure_on_result(self, result: Result) -> Result:
        raise NotImplementedError

    @abstractmethod
    def delay(self):
        raise NotImplementedError



import pytest
import os

from petisco import Petisco
from petisco.event.bus.infrastructure.not_implemented_event_bus import (
    NotImplementedEventBus,
)
from petisco.event.bus.infrastructure.rabbitmq_event_bus import RabbitMqEventBus
from petisco.fixtures.testing_decorators import testing_with_rabbitmq


@pytest.mark.end2end
def test_should_load_petisco_from_yml_and_configure_events_from_yml_with_not_implemented_message_broker(
    petisco_yml_path_end2end
):
    Petisco.clear()

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"
    filename_events = f"{petisco_yml_path_end2end}/petisco.events.not_implemented.yml"

    petisco = Petisco.from_filename(filename)
    petisco.configure_events(filename_events)

    petisco._start()

    assert isinstance(Petisco.get_event_bus(), NotImplementedEventBus)

    petisco.stop()
    Petisco.clear()


@pytest.mark.end2end
@testing_with_rabbitmq
def test_should_load_petisco_from_yml_and_configure_events_from_yml_with_rabbitmq_message_broker(
    petisco_yml_path_end2end
):
    Petisco.clear()

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"
    filename_events = f"{petisco_yml_path_end2end}/petisco.events.yml"

    petisco = Petisco.from_filename(filename)
    petisco.configure_events(filename_events)
    petisco._start()
    assert isinstance(Petisco.get_event_bus(), RabbitMqEventBus)

    assert "config_events" in petisco.info

    petisco.stop()
    Petisco.clear()


@pytest.mark.end2end
@testing_with_rabbitmq
def test_should_load_petisco_from_yml_and_configure_events_from_yml_with_not_implemented_message_broker_but_configured_with_environ(
    petisco_yml_path_end2end
):
    Petisco.clear()

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"
    filename_events = f"{petisco_yml_path_end2end}/petisco.events.not_implemented.yml"
    os.environ["PETISCO_EVENT_MESSAGE_BROKER"] = "rabbitmq"

    petisco = Petisco.from_filename(filename)
    petisco.configure_events(filename_events)

    petisco._start()

    assert isinstance(Petisco.get_event_bus(), RabbitMqEventBus)

    del os.environ["PETISCO_EVENT_MESSAGE_BROKER"]

    petisco.stop()

    Petisco.clear()


@pytest.mark.end2end
@testing_with_rabbitmq
def test_should_load_petisco_from_yml_and_configure_events_from_yml_with_environ_rabbitmq_message_broker_not_implemented(
    petisco_yml_path_end2end
):
    Petisco.clear()
    os.environ["PETISCO_EVENT_MESSAGE_BROKER"] = "notimplemented"

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"
    filename_events = f"{petisco_yml_path_end2end}/petisco.events.yml"

    petisco = Petisco.from_filename(filename)
    petisco.configure_events(filename_events)
    petisco._start()
    assert isinstance(Petisco.get_event_bus(), NotImplementedEventBus)

    del os.environ["PETISCO_EVENT_MESSAGE_BROKER"]

    petisco.stop()
    Petisco.clear()



from typing import Dict

from petisco import IRepository
from tests.end2end.flask_app.shared.infrastructure.repositories.sql_user_repository import (
    SqlUserRepository,
)
from tests.end2end.flask_app.users_count.infrastructure.sql_users_count_repository import (
    SqlUsersCountRepository,
)


def repositories_provider() -> Dict[str, IRepository]:
    return {
        "user": SqlUserRepository.build(),
        "users_count": SqlUsersCountRepository.build(),
    }



import pytest

from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.value_objects.user_id import UserId
from petisco.security.token_decoder.token_decoder import TokenDecoder


@pytest.mark.unit
@pytest.mark.parametrize(
    "type_token,client_id,user_id",
    [
        ("ADMIN_TOKEN", ClientId("client-id"), None),
        ("ADMIN_TOKEN", ClientId("client-id"), UserId("user-id")),
        ("BACKEND_TOKEN", ClientId("client-id"), UserId("user-id")),
    ],
)
def test_should_decode_a_token_successfully(
    type_token, client_id, user_id, given_auth_token_headers_creator
):
    auth_token = given_auth_token_headers_creator(type_token, client_id, user_id)[
        "Authorization"
    ]

    token_decoder = TokenDecoder()
    token = token_decoder.execute(auth_token).unwrap()

    assert token.token_type == type_token
    assert token.client_id == client_id
    assert token.user_id == user_id



from typing import Optional

from dataclasses import dataclass

from petisco.application.config.get_funtion_from_string import get_function_from_string
from petisco.application.config.raise_petisco_config_error import (
    raise_petisco_config_exception,
)


@dataclass
class ConfigLogger:
    selected_logger: Optional[str] = "not_implemented_logger"
    name: Optional[str] = None
    format: Optional[str] = None
    config: Optional[str] = None

    @staticmethod
    def from_dict(kdict):
        config = (
            get_function_from_string(kdict.get("config"))
            .handle(on_failure=raise_petisco_config_exception, failure_args=kdict)
            .unwrap()
        )

        return ConfigLogger(
            selected_logger=kdict.get("selected_logger", "not_implemented_logger"),
            name=kdict.get("name"),
            format=kdict.get("format", "%(name)s - %(levelname)s - %(message)s"),
            config=config,
        )



from petisco import RabbitMqConnector
from petisco.event.bus.infrastructure.rabbitmq_event_bus import RabbitMqEventBus
from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqEventBusMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventBus(connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE)

    @staticmethod
    def with_service(service: str, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventBus(connector, DEFAULT_ORGANIZATION, service)



import os
import random
from time import sleep
from typing import Optional, Dict

from meiga import Failure, Result
from pika.adapters.blocking_connection import BlockingChannel
from pika.spec import Basic

from petisco.event.chaos.domain.event_chaos_error import EventChaosError
from petisco.event.chaos.domain.interface_event_chaos import IEventChaos

EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK_KEY = "EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK"
EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS_KEY = (
    "EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS"
)
EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES_KEY = (
    "EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES"
)


class RabbitMqEventChaos(IEventChaos):
    def __init__(
        self,
        percentage_simulate_nack: Optional[float] = os.environ.get(
            EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK_KEY
        ),
        delay_before_even_handler_second: Optional[float] = os.environ.get(
            EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS_KEY
        ),
        percentage_simulate_failures: Optional[float] = os.environ.get(
            EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES_KEY
        ),
    ):
        """
        Parameters
        ----------
        percentage_simulate_nack
            Percentage of simulate nack [0.0 -> 1.0]. Where 1.0 rejects all the event.
        delay_before_even_handler_second
            Delay event handler execution for a given number of seconds.
        percentage_simulate_failures
            Percentage of simulate failures [0.0 -> 1.0]. Where 1.0 simulate always a failure on handlers.
        """
        self.percentage_simulate_nack = float(percentage_simulate_nack)
        self.delay_before_even_handler_second = float(delay_before_even_handler_second)
        self.percentage_simulate_failures = float(percentage_simulate_failures)

    def info(self) -> Dict:
        return {
            EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK_KEY: self.percentage_simulate_nack,
            EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS_KEY: self.delay_before_even_handler_second,
            EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES_KEY: self.percentage_simulate_failures,
        }

    def nack_simulation(self, ch: BlockingChannel, method: Basic.Deliver):
        if (self.percentage_simulate_nack is None) or (
            random.random() > self.percentage_simulate_nack
        ):
            return False
        ch.basic_nack(delivery_tag=method.delivery_tag)
        return True

    def simulate_failure_on_result(self, result: Result) -> Result:
        if (self.percentage_simulate_failures is None) or (
            random.random() > self.percentage_simulate_failures
        ):
            return result
        else:
            return Failure(EventChaosError(Exception()))

    def delay(self):
        if self.delay_before_even_handler_second is None:
            pass
        else:
            sleep(self.delay_before_even_handler_second)



from meiga import BoolResult

from petisco import Event


def send_mail_on_user_created(event: Event) -> BoolResult:
    pass


def send_referral_on_user_created(event: Event) -> BoolResult:
    pass


def send_promotion_on_user_created(event: Event) -> BoolResult:
    pass


def event_store(event: Event) -> BoolResult:
    pass


def requeue_event(event: Event) -> BoolResult:
    pass



import pytest
from meiga import Success

from petisco import controller_handler, ERROR, InfoId, DEBUG
from petisco.security.token_manager.token_manager import TokenManager
from tests.modules.unit.mocks.fake_logger import FakeLogger
from tests.modules.unit.mocks.log_message_mother import LogMessageMother


@pytest.mark.unit
def test_should_execute_successfully_a_empty_controller_with_jwt_requirement_without_user(
    given_any_petisco,
    given_any_token_type,
    given_any_headers_provider_with_correlation_id,
    given_any_client_id,
    given_any_correlation_id,
):

    logger = FakeLogger()
    expected_info_id = InfoId(
        client_id=given_any_client_id,
        user_id=None,
        correlation_id=given_any_correlation_id,
    )

    @controller_handler(
        logger=logger,
        token_manager=TokenManager(token_type=given_any_token_type),
        headers_provider=given_any_headers_provider_with_correlation_id,
    )
    def my_controller(info_id: InfoId):
        assert info_id == expected_info_id
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Processing Request",
            info_id=expected_info_id,
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Result[status: success | value: Hello Petisco]",
            info_id=expected_info_id,
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_execute_successfully_a_empty_controller_with_jwt_requirement_with_user(
    given_any_petisco,
    given_any_token_type_with_user,
    given_headers_provider,
    given_auth_token_headers_creator,
    given_any_client_id,
    given_any_user_id,
    given_any_correlation_id,
    given_info_id_only_correlation_id,
):

    logger = FakeLogger()
    expected_info_id = InfoId(
        client_id=given_any_client_id,
        user_id=given_any_user_id,
        correlation_id=given_any_correlation_id,
    )

    @controller_handler(
        logger=logger,
        token_manager=TokenManager(
            token_type=given_any_token_type_with_user, require_user=True
        ),
        headers_provider=given_headers_provider(
            {
                **given_auth_token_headers_creator(
                    given_any_token_type_with_user,
                    given_any_client_id,
                    given_any_user_id,
                ),
                **given_info_id_only_correlation_id.get_http_headers(),
            }
        ),
    )
    def my_controller(info_id: InfoId):
        assert info_id == expected_info_id
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Processing Request",
            info_id=expected_info_id,
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Result[status: success | value: Hello Petisco]",
            info_id=expected_info_id,
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_returns_an_error_when_a_empty_controller_do_not_get_a_required_jwt_token(
    given_any_petisco,
    given_other_token_type,
    given_any_info_id,
    given_headers_provider,
    given_auth_token_headers_creator,
    given_any_client_id,
    given_any_user_id,
    given_info_id_only_correlation_id,
):

    logger = FakeLogger()

    @controller_handler(
        logger=logger,
        token_manager=TokenManager(token_type=given_other_token_type),
        headers_provider=given_headers_provider(
            {
                **given_auth_token_headers_creator(
                    "INVALID_TOKEN_TYPE", given_any_client_id, given_any_user_id
                ),
                **given_info_id_only_correlation_id.get_http_headers(),
            }
        ),
    )
    def my_controller():
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == (
        {
            "error": {
                "message": "Access token is missing or invalid. This entry point expects a valid REQUIRED_TOKEN Token",
                "type": "InvalidTokenHttpError",
            }
        },
        401,
    )

    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message == (
        ERROR,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Result[status: failure | value: InvalidTokenError: This entry point expects a valid REQUIRED_TOKEN Token]",
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_returns_an_error_when_a_empty_controller_get_a_required_jwt_token_but_missing_user(
    given_any_petisco,
    given_any_token_type,
    given_headers_provider,
    given_auth_token_headers_creator,
    given_any_token_type_with_user,
    given_any_client_id,
    given_info_id_only_correlation_id,
):
    logger = FakeLogger()

    @controller_handler(
        logger=logger,
        token_manager=TokenManager(token_type=given_any_token_type, require_user=True),
        headers_provider=given_headers_provider(
            {
                **given_auth_token_headers_creator(
                    given_any_token_type_with_user, given_any_client_id, None
                ),
                **given_info_id_only_correlation_id.get_http_headers(),
            }
        ),
    )
    def my_controller():
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == (
        {
            "error": {
                "message": "Access token is missing or invalid. This entry point expects a valid TOKEN Token",
                "type": "InvalidTokenHttpError",
            }
        },
        401,
    )

    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message == (
        ERROR,
        LogMessageMother.get_controller(
            operation="my_controller",
            message="Result[status: failure | value: InvalidTokenError: This entry point expects a valid TOKEN Token]",
        ).to_dict(),
    )



import pytest

from petisco import Event, Name, unique_events


class UserCreated(Event):
    name: Name

    def __init__(self, name: Name):
        self.name = name
        super().__init__()


class VersionedUserCreated(Event):
    name: Name

    def __init__(self, name: Name):
        self.name = name
        super().__init__({"event_version": 2})


@pytest.fixture
def given_any_meta_dict():
    return {"platform_name": "python"}


@pytest.fixture
def given_json_event_data_without_attributes():
    json_data = '{"data":{"meta":{"plataform_name":"iPhone 6s","system_version":"12.4","sdk_version":"1.3.7"},"version":"1","attributes":null,"id":"83D28391-A14E-4122-996D-1A8A63A0B2EA","occurred_on":"2020-07-20 10:14:54.007000","type":"sdk.onboarding.ran"}}'
    return json_data


@pytest.fixture
def given_any_event_dict_with_meta(given_any_name, given_any_meta_dict):
    event_dict = UserCreated(given_any_name).to_dict()
    event_dict["data"]["meta"] = given_any_meta_dict
    return event_dict


@pytest.mark.unit
def test_should_create_an_event_and_check_to_dict_from_dict(
    given_any_info_id, given_any_name
):

    event = UserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)
    event_dict = event.to_dict()

    retrieved_event = UserCreated.from_dict(event_dict)

    assert event == retrieved_event
    assert event.event_info_id == given_any_info_id.to_dict()
    assert event.name == given_any_name


@pytest.mark.unit
def test_should_create_an_event_and_check_to_dict_from_dict_with_non_default_version_event(
    given_any_info_id, given_any_name
):

    event = VersionedUserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)
    event_dict = event.to_dict()

    retrieved_event = UserCreated.from_dict(event_dict)

    assert event == retrieved_event
    assert event.event_info_id == given_any_info_id.to_dict()
    assert event.name == given_any_name


@pytest.mark.unit
def test_should_create_an_event_and_check_its_hash(given_any_info_id, given_any_name):

    event = UserCreated(name=given_any_name)

    hash_before_add_info_id = hash(event)

    event.add_info_id(given_any_info_id)

    hash_after_add_info_id = hash(event)

    assert hash_before_add_info_id != hash_after_add_info_id


@pytest.mark.unit
def test_should_create_an_event_and_check_to_json_from_json(
    given_any_info_id, given_any_name
):

    event = UserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)

    event_json = event.to_json()
    retrieved_event = UserCreated.from_json(event_json)

    assert event == retrieved_event


@pytest.mark.unit
def test_should_create_an_event_and_check_to_json_from_json_with_non_default_version_event(
    given_any_info_id, given_any_name
):

    event = VersionedUserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)

    event_json = event.to_json()
    retrieved_event = UserCreated.from_json(event_json)

    assert event == retrieved_event


@pytest.mark.unit
def test_should_load_an_event_agnostically(given_any_info_id, given_any_name):

    event = UserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)

    event_json = event.to_json()
    agnostic_event = Event.from_json(event_json)

    assert event == agnostic_event


@pytest.mark.unit
def test_should_load_an_event_agnostically_with_non_default_version_event(
    given_any_info_id, given_any_name
):

    event = VersionedUserCreated(name=given_any_name)
    event.add_info_id(given_any_info_id)

    event_json = event.to_json()
    agnostic_event = Event.from_json(event_json)

    assert event == agnostic_event


@pytest.mark.unit
def test_should_check_unique_events():

    user_one = UserCreated(name=Name("one"))
    user_two = UserCreated(name=Name("two"))

    events = unique_events([user_one, user_one, user_one])
    assert len(events) == 1

    events = unique_events([user_one, user_one, user_two, user_two])
    assert len(events) == 2


@pytest.mark.unit
def test_should_update_event_with_info_id_without_deleting_meta_info(
    given_any_info_id, given_any_event_dict_with_meta, given_any_meta_dict
):

    event = Event.from_dict(given_any_event_dict_with_meta)
    assert event.event_meta == given_any_meta_dict

    event.add_info_id(given_any_info_id)

    expected_event_dict = {"info_id": given_any_info_id.to_dict()}
    expected_event_dict.update(given_any_meta_dict)
    assert event.to_dict()["data"]["meta"] == expected_event_dict


@pytest.mark.unit
def test_should_load_json_when_there_is_not_attribute(
    given_json_event_data_without_attributes
):

    event = Event.from_json(given_json_event_data_without_attributes)

    assert not event.to_dict()["data"]["attributes"]



from sqlalchemy import Column, Integer
from petisco.persistence.sqlalchemy.sqlalchemy_persistence import SqlAlchemyPersistence


Base = SqlAlchemyPersistence.get_instance().sources["petisco"]["base"]


class UsersCountModel(Base):
    __tablename__ = "UsersCount"

    count = Column("count", Integer, primary_key=True)



from typing import Optional

from dataclasses import dataclass
from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class PyMongoPersistenceConfig:
    user: Optional[str] = None
    password: Optional[str] = None
    host: Optional[str] = None
    database: Optional[str] = None
    port: Optional[int] = 27017



from typing import Any
from unittest.mock import Mock

import pytest
from meiga import isSuccess, Failure
from pika.adapters.blocking_connection import BlockingChannel
from pika.spec import Basic

from petisco import RabbitMqEventChaos
from petisco.event.chaos.domain.event_chaos_error import EventChaosError


@pytest.mark.integration
def test_should_create_default_rabbitmq_event_chaos():
    chaos = RabbitMqEventChaos()

    assert chaos.nack_simulation(None, None) is False
    chaos.delay()
    assert chaos.simulate_failure_on_result(isSuccess) == isSuccess


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_nack_1():
    chaos = RabbitMqEventChaos(percentage_simulate_nack=1.0)

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.delivery_tag = Mock(Any)

    assert chaos.nack_simulation(mock_channel, mock_method) is True
    chaos.delay()
    assert chaos.simulate_failure_on_result(isSuccess) == isSuccess

    mock_channel.basic_nack.assert_called_once()


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_failures_1():
    chaos = RabbitMqEventChaos(percentage_simulate_failures=1.0)

    assert chaos.nack_simulation(None, None) is False
    chaos.delay()
    assert chaos.simulate_failure_on_result(isSuccess) == Failure(
        EventChaosError(Exception())
    )



from abc import ABCMeta, abstractmethod
from typing import Dict

from meiga import Result, Error, NotImplementedMethodError

from petisco.application.interface_service import IService


class ISumExecutor(IService):
    __metaclass__ = ABCMeta

    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    @abstractmethod
    def execute(self, value1: int, value2: int) -> Result[int, Error]:
        raise NotImplementedMethodError
